@@ -15,6 +15,7 @@
 """Utilities for dealing with AI Platform endpoints API."""
 
 from __future__ import absolute_import
+from __future__ import annotations
 from __future__ import division
 from __future__ import unicode_literals
 
@@ -739,6 +740,7 @@ class EndpointsClient(object):
       accelerator_dict=None,
       min_replica_count=None,
       max_replica_count=None,
+      required_replica_count=None,
       reservation_affinity=None,
       autoscaling_metric_specs=None,
       spot=False,
@@ -747,6 +749,7 @@ class EndpointsClient(object):
       service_account=None,
       traffic_split=None,
       deployed_model_id=None,
+      gpu_partition_size=None,
   ):
     """Deploys a model to an existing endpoint using v1 API.
 
@@ -765,6 +768,8 @@ class EndpointsClient(object):
         deployed model will be always deployed on.
       max_replica_count: int or None, the maximum number of replicas the
         deployed model may be deployed on.
+      required_replica_count: int or None, the required number of replicas the
+        deployed model will be considered successfully deployed.
       reservation_affinity: dict or None, the reservation affinity of the
         deployed model which specifies which reservations the deployed model can
         use.
@@ -778,6 +783,8 @@ class EndpointsClient(object):
         runs as.
       traffic_split: dict or None, the new traffic split of the endpoint.
       deployed_model_id: str or None, id of the deployed model.
+      gpu_partition_size: str or None, the partition size of the GPU
+        accelerator.
 
     Returns:
       A long-running operation for DeployModel.
@@ -804,6 +811,8 @@ class EndpointsClient(object):
         machine_spec.reservationAffinity = flags.ParseReservationAffinityFlag(
             reservation_affinity, constants.GA_VERSION
         )
+      if gpu_partition_size is not None:
+        machine_spec.gpuPartitionSize = gpu_partition_size
 
       dedicated = self.messages.GoogleCloudAiplatformV1DedicatedResources(
           machineSpec=machine_spec, spot=spot
@@ -813,6 +822,8 @@ class EndpointsClient(object):
       dedicated.minReplicaCount = min_replica_count or 1
       if max_replica_count is not None:
         dedicated.maxReplicaCount = max_replica_count
+      if required_replica_count is not None:
+        dedicated.requiredReplicaCount = required_replica_count
 
       if autoscaling_metric_specs is not None:
         autoscaling_metric_specs_list = []
@@ -889,6 +900,7 @@ class EndpointsClient(object):
       accelerator_dict=None,
       min_replica_count=None,
       max_replica_count=None,
+      required_replica_count=None,
       reservation_affinity=None,
       autoscaling_metric_specs=None,
       spot=False,
@@ -898,10 +910,12 @@ class EndpointsClient(object):
       traffic_split=None,
       deployed_model_id=None,
       shared_resources_ref=None,
+      *,
       min_scaleup_period=None,
       idle_scaledown_period=None,
       initial_replica_count=None,
       gpu_partition_size=None,
+      min_gpu_driver_version: str | None = None,
   ):
     """Deploys a model to an existing endpoint using v1beta1 API.
 
@@ -920,6 +934,8 @@ class EndpointsClient(object):
         deployed model will be always deployed on.
       max_replica_count: int or None, the maximum number of replicas the
         deployed model may be deployed on.
+      required_replica_count: int or None, the required number of replicas the
+        deployed model will be considered successfully deployed.
       reservation_affinity: dict or None, the reservation affinity of the
         deployed model which specifies which reservations the deployed model can
         use.
@@ -946,6 +962,8 @@ class EndpointsClient(object):
         enrolled in scale-to-zero.
       gpu_partition_size: str or None, the partition size of the GPU
         accelerator.
+      min_gpu_driver_version: str or None, the minimum GPU driver version that
+        this machine requires.
 
     Returns:
       A long-running operation for DeployModel.
@@ -992,6 +1010,8 @@ class EndpointsClient(object):
           )
         if gpu_partition_size is not None:
           machine_spec.gpuPartitionSize = gpu_partition_size
+        if min_gpu_driver_version is not None:
+          machine_spec.minGpuDriverVersion = min_gpu_driver_version
 
         dedicated = (
             self.messages.GoogleCloudAiplatformV1beta1DedicatedResources(
@@ -1011,6 +1031,8 @@ class EndpointsClient(object):
         else:
           if max_replica_count is not None:
             dedicated.maxReplicaCount = max_replica_count
+        if required_replica_count is not None:
+          dedicated.requiredReplicaCount = required_replica_count
 
         # if not specified and min-replica-count is 0, default to 1.
         if initial_replica_count is None and dedicated.minReplicaCount == 0:
