@@ -35,6 +35,11 @@ from googlecloudsdk.generated_clients.apis.gkehub.v1alpha import gkehub_v1alpha_
 _BINAUTHZ_GKE_POLICY_REGEX = (
     'projects/([^/]+)/platforms/gke/policies/([a-zA-Z0-9_-]+)'
 )
+_STRAGGLER_MIGRATION_STRATEGY_ENUMS = (
+    'STRAGGLER_MIGRATION_STRATEGY_UNSPECIFIED',
+    'STRAGGLER_MIGRATION_STRATEGY_NO_MIGRATION',
+    'STRAGGLER_MIGRATION_STRATEGY_LAST_WAVE',
+)
 
 
 # TODO(b/312311133): Deduplicate shared code between fleet and rollout commands.
@@ -60,6 +65,41 @@ class RolloutFlags:
   def AddAsync(self):
     base.ASYNC_FLAG.AddToParser(self.parser)
 
+  def AddCustomWaves(self):
+    self.parser.add_argument(
+        '--custom-wave',
+        type=arg_parsers.ArgDict(
+            spec={
+                'upper-bound-percentage': arg_parsers.BoundedInt(
+                    lower_bound=0, upper_bound=100
+                ),
+                'minimum-completion-percentage': arg_parsers.BoundedInt(
+                    lower_bound=0, upper_bound=100
+                ),
+                'minimum-completion-number': arg_parsers.BoundedInt(
+                    lower_bound=0
+                ),
+                'straggler-migration-strategy': str,
+            },
+            required_keys=[],
+        ),
+        action='append',
+        help=textwrap.dedent("""\
+            Comma-separated list of custom waves to be used for the rollout.
+            Each custom wave is specified as `upper-bound-percentage`,
+            `minimum-completion-percentage` or `minimum-completion-number`, and
+            `straggler-migration-strategy`.
+
+            In order to have multiple custom waves, use the `--custom-wave`
+            flag multiple times.
+
+            `straggler-migration-strategy` can be one of:
+            - 'STRAGGLER_MIGRATION_STRATEGY_UNSPECIFIED',
+            - 'STRAGGLER_MIGRATION_STRATEGY_NO_MIGRATION',
+            - 'STRAGGLER_MIGRATION_STRATEGY_LAST_WAVE',
+        """),
+    )
+
   def AddDisplayName(self):
     self.parser.add_argument(
         '--display-name',
@@ -70,6 +110,38 @@ class RolloutFlags:
         """),
     )
 
+  def AddExcludeMembershipNames(self):
+    self.parser.add_argument(
+        '--exclude-membership-names',
+        type=arg_parsers.ArgList(),
+        metavar='MEMBERSHIP_NAME',
+        help=textwrap.dedent("""\
+            Comma-separated list of full names of memberships to exclude
+            from the rollout. The memberships must use the following format:
+
+            `projects/<PROJECT_NUMBER>/locations/global/memberships/<MEMBERSHIP_ID>`.
+
+            Either this flag or `--include-membership-names` can be specified,
+            but not both.
+        """),
+    )
+
+  def AddIncludeMembershipNames(self):
+    self.parser.add_argument(
+        '--include-membership-names',
+        type=arg_parsers.ArgList(),
+        metavar='MEMBERSHIP_NAME',
+        help=textwrap.dedent("""\
+            Comma-separated list of full names of memberships to include in
+            the rollout. The memberships must use the following format:
+
+            `projects/<PROJECT_NUMBER>/locations/global/memberships/<MEMBERSHIP_ID>`.
+
+            Either this flag or `--excluded-membership-names` can be specified,
+            but not both.
+        """),
+    )
+
   def AddLabels(self):
     self.parser.add_argument(
         '--labels',
@@ -122,9 +194,11 @@ class RolloutFlags:
         '--soak-duration',
         help=textwrap.dedent("""\
           Soak time before starting the next wave. e.g. `4h`, `2d6h`.
+          Soak duration is required for Rollouts.
 
           See $ gcloud topic datetimes for information on duration formats."""),
         type=arg_parsers.Duration(),
+        required=True,
     )
 
   def AddRolloutResourceArg(self):
@@ -135,9 +209,7 @@ class RolloutFlags:
 
   def AddRolloutTypeConfig(self):
     rollout_type_mutex_group = self.parser.add_mutually_exclusive_group(
-        help=(
-            'Configuration for specific rollout types.'
-        ),
+        help='Configuration for specific rollout types.',
     )
     self._AddVersionUpgrade(rollout_type_mutex_group)
     self._AddFeatureUpdate(rollout_type_mutex_group)
@@ -148,9 +220,21 @@ class RolloutFlags:
     version_upgrade_config_group = rollout_type_mutex_group.add_group(
         help='Version upgrade rollout config.',
     )
-    self._AddControlPlaneUpgrade(version_upgrade_config_group)
+    self._AddVersionUpgradeType(version_upgrade_config_group)
     self._AddTargetVersion(version_upgrade_config_group)
 
+  def _AddVersionUpgradeType(
+      self, rollout_type_mutex_group: parser_arguments.ArgumentInterceptor
+  ):
+    version_upgrade_type_mutex_group = (
+        rollout_type_mutex_group.add_mutually_exclusive_group(
+            help='Version upgrade type to use for the Rollout.',
+        )
+    )
+
+    self._AddControlPlaneUpgrade(version_upgrade_type_mutex_group)
+    self._AddConfigSyncUpgrade(version_upgrade_type_mutex_group)
+
   def _AddControlPlaneUpgrade(
       self, version_upgrade_config_group: parser_arguments.ArgumentInterceptor
   ):
@@ -161,6 +245,16 @@ class RolloutFlags:
         action='store_true',
     )
 
+  def _AddConfigSyncUpgrade(
+      self, version_upgrade_config_group: parser_arguments.ArgumentInterceptor
+  ):
+    version_upgrade_config_group.add_argument(
+        '--config-sync-upgrade',
+        help=textwrap.dedent("""\
+          Upgrade the config sync of the clusters in the fleet."""),
+        action='store_true',
+    )
+
   def _AddTargetVersion(
       self, version_upgrade_config_group: parser_arguments.ArgumentInterceptor
   ):
@@ -169,6 +263,7 @@ class RolloutFlags:
         help=textwrap.dedent("""\
           The version to upgrade clusters from the fleet to."""),
         type=str,
+        required=True,
     )
 
   def _AddFeatureUpdate(
@@ -350,8 +445,82 @@ class RolloutFlagParser:
     return self.args.start_time.strftime('%Y-%m-%dT%H:%M:%SZ')
 
   def _ManagedRolloutConfig(self) -> fleet_messages.ManagedRolloutConfig:
+    """Constructs message ManagedRolloutConfig.
+
+    Parses arguments related to the managed rollout configuration, such as
+    `--soak-duration` and `--exclude-membership-names`.
+
+    Returns:
+      A fleet_messages.ManagedRolloutConfig instance or None if no related
+      flags were specified.
+    """
     managed_rollout_config = fleet_messages.ManagedRolloutConfig()
     managed_rollout_config.soakDuration = self._SoakDuration()
+
+    uipr_rollout_config = fleet_messages.UIPRRolloutConfig()
+
+    if (
+        '--exclude-membership-names' in self.args.GetSpecifiedArgs()
+        and '--include-membership-names' in self.args.GetSpecifiedArgs()
+    ):
+      raise ValueError(
+          '`--exclude-membership-names` and `--include-membership-names` cannot'
+          ' be set at the same time.'
+      )
+
+    if '--exclude-membership-names' in self.args.GetSpecifiedArgs():
+      for membership in self.args.exclude_membership_names:
+        uipr_rollout_config.excludedMembershipNames.append(membership)
+
+    if '--include-membership-names' in self.args.GetSpecifiedArgs():
+      for membership in self.args.include_membership_names:
+        uipr_rollout_config.includeMembershipNames.append(membership)
+
+    if '--custom-wave' in self.args.GetSpecifiedArgs():
+      custom_waves = []
+      for wave in self.args.custom_wave:
+        if ('minimum-completion-percentage' in wave) and (
+            'minimum-completion-number' in wave
+        ):
+          raise ValueError(
+              'Exactly one of `minimum-completion-percentage` or '
+              '`minimum-completion-number` must be specified for each wave in '
+              '--custom-wave.'
+          )
+        custom_wave = fleet_messages.WaveTemplate()
+        if 'upper-bound-percentage' in wave:
+          custom_wave.upperBoundPercentage = wave['upper-bound-percentage']
+
+        if 'minimum-completion-percentage' in wave:
+          custom_wave.minimumCompletionPercentage = wave[
+              'minimum-completion-percentage'
+          ]
+
+        if 'minimum-completion-number' in wave:
+          custom_wave.minimumCompletionNumber = wave[
+              'minimum-completion-number'
+          ]
+
+        if 'straggler-migration-strategy' in wave:
+          if (
+              wave['straggler-migration-strategy']
+              not in _STRAGGLER_MIGRATION_STRATEGY_ENUMS
+          ):
+            raise ValueError(
+                'Invalid value for `straggler-migration-strategy`:'
+                f' {wave["straggler-migration-strategy"]}. Must be one of'
+                f' {_STRAGGLER_MIGRATION_STRATEGY_ENUMS}.'
+            )
+          custom_wave.stragglerMigrationStrategy = fleet_messages.WaveTemplate.StragglerMigrationStrategyValueValuesEnum(
+              wave['straggler-migration-strategy']
+          )
+
+        custom_waves.append(custom_wave)
+
+      uipr_rollout_config.customWaves = custom_waves
+
+    managed_rollout_config.uiprRolloutConfig = uipr_rollout_config
+
     return self.TrimEmpty(managed_rollout_config)
 
   def _SoakDuration(self) -> str:
@@ -377,11 +546,21 @@ class RolloutFlagParser:
   def _VersionUpgradeType(
       self,
   ) -> fleet_messages.VersionUpgrade.TypeValueValuesEnum:
-    """Parses --control-plane-upgrade."""
+    """Parses --control-plane-upgrade and --config-sync-upgrade."""
     enum_type = fleet_messages.VersionUpgrade.TypeValueValuesEnum
-    if '--control-plane-upgrade' not in self.args.GetSpecifiedArgs():
-      return enum_type.TYPE_UNSPECIFIED
-    return enum_type.TYPE_CONTROL_PLANE
+    if (
+        '--control-plane-upgrade' in self.args.GetSpecifiedArgs()
+        and '--config-sync-upgrade' in self.args.GetSpecifiedArgs()
+    ):
+      raise ValueError(
+          '--control-plane-upgrade and --config-sync-upgrade cannot be set at'
+          ' the same time.'
+      )
+    if '--control-plane-upgrade' in self.args.GetSpecifiedArgs():
+      return enum_type.TYPE_CONTROL_PLANE
+    if '--config-sync-upgrade' in self.args.GetSpecifiedArgs():
+      return enum_type.TYPE_CONFIG_SYNC
+    return enum_type.TYPE_UNSPECIFIED
 
   def _VersionUpgradeDesiredVersion(self) -> str:
     """Parses --target-version."""
